﻿/*
 * Simple image processing library
 * Inspiration: http://www.switchonthecode.com/tutorials/csharp-tutorial-image-editing-saving-cropping-and-resizing
 */
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;

namespace LogonChanger
{
    public class ImageProcessor
    {
        /// <summary>
        /// As the name suggests.
        /// </summary>
        /// <param name="imgToResize"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        public static Image resizeImage(Image imgToResize, Size size)
        {
            Image b = new Bitmap(size.Width, size.Height);
            using (Graphics g = Graphics.FromImage(b))
            {
                g.InterpolationMode = InterpolationMode.HighQualityBicubic;

                g.DrawImage(imgToResize, 0, 0, size.Width, size.Height);
            }

            return b;
        }

        /// <summary>
        /// Crops image. What else?
        /// </summary>
        /// <param name="img"></param>
        /// <param name="cropArea"></param>
        /// <returns></returns>
        public static Image cropImage(Image img, Rectangle cropArea)
        {
            var bmpImage = new Bitmap(img);
            Image cropped = bmpImage.Clone(cropArea, img.PixelFormat);
            return (cropped);
        }

        /// <summary>
        /// Compresses to JPG with the highest quality that will
        /// result in an image size less than targetSize.
        /// </summary>
        /// <param name="image"></param>
        /// <param name="targetDimension"></param>
        /// <param name="targetSizeBytes"></param>
        /// <returns></returns>
        public static JpgImage compressToJpeg(Image image, Size targetDimension, long targetSizeBytes)
        {
            var hi = new JpgImage();
            var lo = new JpgImage();
            hi.Quality = 100;
            lo.Quality = 0;
            var curr = hi;
            //var last = lo;


            // Resizing returns image witn 96 DPI
            using (Image img = resizeImage(image, targetDimension))
            {
                // Binary Search.
                // Jpg Quality 0-100, max iterations: 7
                do
                {
                    curr = new JpgImage();
                    curr.Quality = lo.Quality + (hi.Quality - lo.Quality) / 2;

                    curr.writeImageToStream(img);
                    if (curr.Size > targetSizeBytes)
                    {
                        hi.Dispose();
                        hi = curr;
                    }
                    else
                    {
                        lo.Dispose();
                        lo = curr;
                    }
                } while (Math.Abs(hi.Quality - lo.Quality) > 1);
            }// using

            // If ( T >= H > L ) or ( L > T >= C ) then use C
            // otherwise, use L
            if (targetSizeBytes >= hi.Size && hi.Size > lo.Size)
            {
                lo.Dispose();
                return hi;
            }
            else
            {
                hi.Dispose();
                return lo;
            }
        }// end compressor
    }
        
    /// <summary>
    /// Custom class to store some extra image information
    /// </summary>
    public class JpgImage : IDisposable
    {
        private readonly MemoryStream _Stream = new MemoryStream();
        
        public int Quality { get; set; }

        public long Size{get; set;}

        public MemoryStream Stream
        {
            get { return _Stream; }
        }

        public Image Image
        {
            get { return Image.FromStream(_Stream); }
        }

        #region IDisposable Members

        public void Dispose()
        {
            _Stream.Dispose();
            Size = 0;
        }

        #endregion

        public void writeStreamToFile(string path, bool createBackup)
        {
            var fInfo = new FileInfo(path);
            if (!fInfo.Directory.Exists)
                fInfo.Directory.Create();

            if (createBackup && fInfo.Exists)
            {
                fInfo.CopyTo(fInfo.FullName + ".bak", true);
            }
            using (var file = new FileStream(path, FileMode.Create))
            {
                _Stream.WriteTo(file);
                file.Flush();
            }
        }

        public void writeImageToStream(Image img)
        {
            // Jpeg image codec
            ImageCodecInfo jpegCodec = getEncoderInfo("image/jpeg");

            if (jpegCodec == null)
                return;

            // Encoder parameter for image quality & 24bit color depth
            var encoderParams = new EncoderParameters(2);
            encoderParams.Param[0] = new EncoderParameter(Encoder.Quality, Quality);
            encoderParams.Param[1] = new EncoderParameter(Encoder.ColorDepth, 24L);

            img.Save(Stream, jpegCodec, encoderParams);
            Size = Stream.Length;
        }

        private static ImageCodecInfo getEncoderInfo(string mimeType)
        {
            // Get image codecs for all image formats
            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();

            // Find the correct image codec
            for (int i = 0; i < codecs.Length; i++)
                if (codecs[i].MimeType == mimeType)
                    return codecs[i];
            return null;
        }
    }
}